﻿//////////////////////////////////////////////
// ProgramSourcesHolder.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// nkMemory
#include <NilkinsMemory/Containers/StringView.h>

// Standards
#include <string>
#include <vector>

/// Internals --------------------------------

namespace nkGraphics
{
	struct DLL_GRAPHICS_EXPORT MacroEntry
	{
		std::string _key ;
		std::string _value ;
	} ;
}

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT ProgramSourcesHolder final
	{
		private :

			// Les mémoires
			// Ce seront des char* pour éviter les différences 
			std::string _vertexMemory ;
			std::string _hullMemory ;
			std::string _domainMemory ;
			std::string _geometryMemory ;
			std::string _pixelMemory ;
			std::string _computeMemory ;
			std::string _raytracingMemory ;

			std::vector<MacroEntry> _macrosVertex ;
			std::vector<MacroEntry> _macrosHull ;
			std::vector<MacroEntry> _macrosDomain ;
			std::vector<MacroEntry> _macrosGeometry ;
			std::vector<MacroEntry> _macrosPixel ;
			std::vector<MacroEntry> _macrosCompute ;
			std::vector<MacroEntry> _macrosRaytracing ;

		public :

			// Constructeur, destructeur
			ProgramSourcesHolder () ;
			~ProgramSourcesHolder () ;

			// Getters
			nkMemory::StringView getVertexMemory () const ;
			nkMemory::StringView getHullMemory () const ;
			nkMemory::StringView getDomainMemory () const ;
			nkMemory::StringView getGeometryMemory () const ;
			nkMemory::StringView getPixelMemory () const ;
			nkMemory::StringView getComputeMemory () const ;
			nkMemory::StringView getRaytracingMemory () const ;
			const std::vector<MacroEntry>& getDefineMacrosVertex () const ;
			const std::vector<MacroEntry>& getDefineMacrosHull () const ;
			const std::vector<MacroEntry>& getDefineMacrosDomain () const ;
			const std::vector<MacroEntry>& getDefineMacrosGeometry () const ;
			const std::vector<MacroEntry>& getDefineMacrosPixel () const ;
			const std::vector<MacroEntry>& getDefineMacrosCompute () const ;
			const std::vector<MacroEntry>& getDefineMacrosRaytracing () const ;
			unsigned int getDefineMacroMaxNumber () const ;
			unsigned int getDefineMacroMaxNumberPipeline () const ;
			unsigned int getDefineMacroNumberCompute () const ;
			unsigned int getDefineMacroNumberRaytracing () const ;

			// Setters
			void setVertexMemory (const nkMemory::StringView& str) ;
			void setHullMemory (const nkMemory::StringView& str) ;
			void setDomainMemory (const nkMemory::StringView& str) ;
			void setGeometryMemory (const nkMemory::StringView& str) ;
			void setPixelMemory (const nkMemory::StringView& str) ;
			void setComputeMemory (const nkMemory::StringView& str) ;
			void setRaytracingMemory (const nkMemory::StringView& str) ;
			void addDefineMacroVertex (const nkMemory::StringView& key, const nkMemory::StringView& value) ;
			void addDefineMacroHull (const nkMemory::StringView& key, const nkMemory::StringView& value) ;
			void addDefineMacroDomain (const nkMemory::StringView& key, const nkMemory::StringView& value) ;
			void addDefineMacroGeometry (const nkMemory::StringView& key, const nkMemory::StringView& value) ;
			void addDefineMacroPixel (const nkMemory::StringView& key, const nkMemory::StringView& value) ;
			void addDefineMacroCompute (const nkMemory::StringView& key, const nkMemory::StringView& value) ;
			void addDefineMacroRaytracing (const nkMemory::StringView& key, const nkMemory::StringView& value) ;

			// Utilitaires
			bool hasMinimalDefinitionsPipeline () const ;
			bool hasMinimalDefinitionsCompute () const ;
			bool hasMinimalDefinitionsGeometry () const ;
			bool hasMinimalDefinitionsRaytracing () const ;
			void copyDefinesFrom (const ProgramSourcesHolder& other) ;
	} ;
}